//author: Vladimir MV Kish ,Novi Sad, 2016
/*

Problem starting point is a list of edges, with two indices of nodes. (example 1-2 , 5-6 , 8-9 , 2-3)
Edges can't contain reflective nodes. (example 1-1 , 5-5).
Edges can't contain simmetric nodes.  (example 1-2 2-1 , 3-4 4-3)

Edges are cliques of size 2. If we have two edges that contain a same node,
they will make a clique of size 3 if there is another edge that connects nodes that are not common to these two edges.

This can be generalized for all cliques, that share all nodes except one (one per clique, two total).

If a clique has all same nodes as another clique except one(two), than they will form a bigger clique if
there is one edge that connects the nodes that are not shared. 

Algorithm first finds all the cliques of size three (3).
If there are more than one found, then it searches for cliques of size four (4).
If there are more than one found, then it searches for cliques of size five (5), and so on...

Clique is represented as a list of all nodes that make it. (std::vector in c++)
If lists of nodes of two cliques differ only in one place (two nodes) , search is performed inside all other cliques for an edge that connects those two nodes.

The result is a list of cliques of different sizes.

Only the maximum size cliques are all present in the result.
Some of smaller cliques are implied by bigger cliques.
If there are more small cliques than the size of result list (number of edges), than they imply that there must be a bigger clique.

The algorithm contains four nested loops dependant on number of input edges, so the complexity is about O(n^4) .
Number of elements in the result is same as input number of elements, but elements (cliques) in the result sizes are bigger.

Clique problem relates to P versus NP problem.
*/


#include"QuickClique.h"

namespace main2{
	
	extern std::vector< std::vector<unsigned int> > rez;
	extern int gsizelimit;

	//does clique a contain clique b (has all the nodes that b has ,but may have more)
	int contains(std::vector<unsigned int>* a,std::vector<unsigned int>* b){ 
			if(a->size()<b->size())return 0;

			unsigned int i,j;

			unsigned int eq=0;

			for(i=0;i<a->size();i++){
			for(j=0;j<b->size();j++){
				if((*a)[i]==(*b)[j])eq++;
			}}

			if(eq<b->size())return 0;
			return 1;
		}

		//do cliques a and b differ only in one member ?
		int compare(unsigned int* ii,unsigned int* jj,std::vector<unsigned int>* a,std::vector<unsigned int>* b){ 
			if(a->size()!=b->size())return 0;

			unsigned int sz= a->size();

			unsigned int i,j;

			unsigned int m1=(unsigned int)-1;

			int eq=0;
			std::vector<unsigned int> aa=*a,bb=*b;
			for(i=0;i<sz;i++){
			for(j=0;j<sz;j++){
				if((*a)[i]==(*b)[j]){aa[i]=m1;bb[j]=m1;eq++;}
			}}

			if(eq != sz-1)return 0;

			i=0;
			while(i<sz && aa[i]==m1 )
				i++;

			if(i>=sz)return 0;

			j=0;
			while(j<sz && bb[j]==m1 )
				j++;

			if(j>=sz)return 0;

			*ii=aa[i];
			*jj=bb[j];

			if(*ii==*jj)return 0;
			return 1;
		}

		// does a clique contain an edge ii-jj
		int findedge(unsigned int ii,unsigned int jj, std::vector<unsigned int>* a){  
			unsigned int rez=0;
			for(unsigned int i=0;i<a->size();i++) {if((*a)[i]==ii) rez++;}
			for(unsigned int i=0;i<a->size();i++) {if((*a)[i]==jj) rez++;}
			if(rez>=2)return 1;
			return 0;
		}


		//find all (sizes) cliques //edges must not contain reflective or symetric nodes 
		void findall(std::vector<std::vector<unsigned int>>* edges){ 

			int t=GetTickCount();
			int t3, t2=t;

			rez.clear();

			rez=*edges;
			unsigned int ii,jj;

			std::vector<unsigned int> rezi;

			int n=2;
			unsigned int m=0;
			int count=0;

			do{
				count=0;

					for(unsigned int i=0; i<rez.size()-1;i++){
					for(unsigned int j=i+1; j<rez.size();j++){
						if(!compare(&ii,&jj,&(rez[i]),&(rez[j])))continue;
						for(unsigned int k=0; k<rez.size();k++){
							if(!findedge(ii,jj,&(rez[k])))continue;
							rezi=rez[i];
							rezi.push_back(jj);
							int cont=0;
							for(m=0;m<rez.size();m++){
								if(contains(&(rez[m]),&rezi)){
									cont=1;
									break;
								}
							}
							if(cont==0){
								rez[i].push_back(jj);
								count++;
								break;
							}
						}
					}}
					

					n++;

					printf("\nCurrent size= %d , time= %d ms",n,(t3=GetTickCount())-t2); t2=t3;

			}while(count>1 && (n<gsizelimit || gsizelimit<0));

		
			printf("\ntotal time= %d ms",GetTickCount()-t);
		}

}


int WINAPI WinMain(HINSTANCE hInst,HINSTANCE hPrevInst,LPSTR lpCmdLine,int nCmdShow){
	LibWinMain(hInst,hPrevInst,lpCmdLine,nCmdShow);
}